/*
 * Copyright (c) 2002
 *	Politecnico di Torino.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the Politecnico
 * di Torino, and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @(#) $Header: /usr/cvsroot/wpdpack/Include/pcap-stdinc.h,v 1.5 2003/04/04 11:15:08 degioanni Exp $ (LBL)
 */

#define SIZEOF_CHAR  1
#define SIZEOF_SHORT 2
#define SIZEOF_INT   4

#include <fcntl.h>

#include <winsock2.h>
#include "bittypes.h"
#include <time.h>
#include <io.h>

#if !defined(__MINGW32__)
#include "ip6_misc.h"
#endif

#define caddr_t char*
