/** \file   settings_fsdevice.h
 * \brief   File system device settings - header
 *
 * \author  Bas Wassink <b.wassink@ziggo.nl>
 */


#ifndef VICE_SETTINGS_FSDEVICE_H
#define VICE_SETTINGS_FSDEVICE_H


GtkWidget *settings_fsdevice_widget_create(GtkWidget *parent);

#endif
