/*
 * Globals.java
 *
 * Written by
 *  Locnet <android.locnet@gmail.com>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

package com.locnet.vice;

public class Globals {
    public static String PREFKEY_ROM = "rom_location";
    public static int PREFKEY_ROM_INT = 0;

    public static String PREFKEY_F1 = "f1_location";
    public static int PREFKEY_F1_INT = 1;

    public static String PREFKEY_F2 = "f2_location";
    public static int PREFKEY_F2_INT = 2;

    public static String PREFKEY_F3 = "f3_location";
    public static int PREFKEY_F3_INT = 3;

    public static String PREFKEY_F4 = "f4_location";
    public static int PREFKEY_F4_INT = 4;
}
